# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, email, imaplib, time, excepts

def fix_text(text):
    """
    Elimina los saltos de línea de una cadena de texto y sustituye cada
    tabulador por un espacio en blanco.

    `text` es la cadena de texto a tratar.
    """

    newText = text.replace("\n", "")
    newText = newText.replace("\t", " ")
    return newText
    
class EmailAttachment:
    """
    Clase que representa un archivo adjunto de un mensaje de e-mail de un servidor de IMAP.
    """

    def __init__(self, content, fileName):
        """
        Constructor de la clase.

        `content` es el contenido del archivo.

        `fileName` es el nombre del archivo.
        """

        self.content = content
        self.fileName = fix_text(fileName)
    
    def save(self, dirPath, fileName=None):
        """
        Guarda el archivo.

        `dirPath` es la ruta del directorio donde guardar el archivo.

        `fileName` es el nombre del archivo que se va a establecer. Si no se especifica, el nombre 
        del archivo es su nombre original.
        """

        if (fileName == None):
            path = os.path.join(dirPath, self.fileName)
        else:
            path = os.path.join(dirPath, fileName)
        outputFile = open(path, "wb")
        outputFile.write(self.content)
        outputFile.close()
        
class EmailAttachmentSet:
    """
    Conjunto de los archivos adjuntos de un mensaje concreto de un servidor de IMAP.
    Es una colección de objetos de tipo `EmailAttachment`.
    """

    def __init__(self, emailMsgContent):
        """
        Constructor de la clase.

        `emailMsgContent` es el contenido del mensaje.
        """

        self.attachmentsIndex = 0 
        self.attachments = []
    
        for part in emailMsgContent.walk():
            fileName = part.get_filename()
            if fileName != None:
                fileContent = part.get_payload(decode=1)
                self.attachments.append(EmailAttachment(fileContent, fileName))
    
    def count(self):
        """
        Devuelve el número de archivos adjuntos del mensaje.
        """

        return len(self.attachments)
        
    def __iter__(self):
        """
        Método necesario para que la clase sea iterable.

        Se itera sobre los archivos adjuntos.
        """

        return self

    def next(self):
        """
        Método necesario para que la clase sea iterable.

        Se itera sobre los archivos adjuntos.

        Devuelve el siguiente archivo adjunto (objeto de tipo `EmailAttachment`).
        """

        try: 
            attachment = self.attachments[self.attachmentsIndex]
        except:
            self.attachmentsIndex = 0
            raise StopIteration
        self.attachmentsIndex += 1
        return attachment
        
class EmailMessage:
    """
    Clase que representa un mensaje de e-mail completo de un servidor de IMAP.
    """

    def __init__(self, index, data):
        """
        Constructor de la clase.
        
        `index` es el índice del mensaje dentro del contexto del servidor de e-mail.

        `data` es el contenido del mensaje.
        """

        self.index = index
        self.content = email.message_from_string(data[0][1])
        
        self.subject = self.content["subject"]
        self.fromAdd = self.content["from"]
        self.attachments = EmailAttachmentSet(self.content)
    
    def __cmp__(self, other):
        """
        Método comparador de la clase.

        Es necesario para saber si un mensaje es distinto de otro. El criterio de
        comparación es por su asunto.
        """

        if self.subject == other.subject:
            result = 0
        elif self.subject < other.subject:
            result = -1
        else:
            result = 1
        return result

class EmailMessagePreview:
    """
    Clase que representa las cabeceras de un mensaje de e-mail de un servidor de IMAP.
    """

    def __init__(self, index, data):
        """
        Constructor de la clase.

        `index` es el índice del mensaje dentro del contexto del servidor de e-mail.

        `data` es el contenido de las cabeceras.
        """

        self.index = index
        self.content = email.message_from_string(data[0][1])

        self.subject = self.content["subject"]
        self.fromAdd = self.content["from"]
    
    def __cmp__(self, other):
        """
        Método comparador de la clase.

        Es necesario para saber si un mensaje es distinto de otro. El criterio de
        comparación es por su asunto.
        """

        if self.subject == other.subject:
            result = 0
        elif self.subject < other.subject:
            result = -1
        else:
            result = 1
        return result

class ImapAccount:
    """
    Clase básica para acceder a una cuenta de un servidor de IMAP y descargar y borrar mensajes de e-mail.
    """

    def __init__(self, server, ssl, port, user, password):
        """
        Constructor de la clase.

        `server` es la dirección del servidor con el que conectar.

        `ssl` es una variable de tipo lógico que especifica si se debe usar una conexión
        segura por medio de SSL (depende del servidor).

        `port` es el puerto a usar para realizar la conexión (depende del servidor).

        `user` es el nombre de usuario con el que conectarse al servidor.

        `password` es la contraseña del usuario `user`.
        """

        self.server = server
        self.ssl = ssl
        self.port = port
        self.user = user
        self.password = password
        self.messageCount = None
        
    def connect(self):
        """
        Realiza la conexión con el servidor.
        """

        try:
            if self.ssl:
                self.connection = imaplib.IMAP4_SSL(self.server, self.port)
            else:
                self.connection = imaplib.IMAP4(self.server, self.port)
        except:
            self.close()
            raise excepts.ConnectionException()
        
        try:
            self.connection.login(self.user, self.password)
        except:
            self.close()
            raise excepts.LoginException()
            
        self.refresh()

    def close(self):
        """
        Cierra la conexión con el servidor.

        Para llamar a este método, debe haber una conexión abierta.
        """

        if self.connection != None:
            self.connection.close()
            self.connection.logout()
            self.connection = None
    
    def refresh(self):
        """
        Actualiza el contador de mensajes de la bandeja de entrada del servidor.
        """

        typ, data = self.connection.select("INBOX")
        self.messageCount = int(data[0])
        
    def get_messages_count(self):
        """
        Devuelve el número de mensajes de la cuenta.

        Para llamar a este método, antes debe llamarse a `refresh`.
        """

        return self.messageCount

    # Returns a list of message index
    def get_messages_by_subject(self, criteria):
        """
        Devuelve una lista con los índices de todos
        los mensajes cuyo asunto contenga un texto concreto.

        `criteria` es el texto a usar en la búsqueda.
        """

        param = '(SUBJECT "' + criteria + '")'
        typ, [ids] = self.connection.search(None, param)
        idsInt = ids.split()
        for i in range(0, len(idsInt)):
            idsInt[i] = int(idsInt[i])
        return idsInt

    # "index" starts at 1
    def get_message(self, index):
        """
        Devuelve un objeto de tipo `imap.EmailMessage`, representando a un mensaje
        de e-mail concreto.

        `index` es el índice del mensaje a obtener. Debe ser mayor o igual que 1.    
        """

        typ, data = self.connection.fetch(str(index), "(RFC822)")
        return EmailMessage(index, data)
        
    # "index" starts at 1
    def get_message_preview(self, index):
        """
        Devuelve un objeto de tipo `imap.EmailMessagePreview`, representando a un mensaje
        de e-mail concreto.

        `index` es el índice del mensaje a obtener. Debe ser mayor o igual que 1.    
        """

        typ, data = self.connection.fetch(str(index), "(BODY.PEEK[HEADER])")
        return EmailMessagePreview(index, data)
    
    # "index" starts at 1
    def remove_message(self, index):
        """
        Borra un mensaje de e-mail concreto.

        `index` es el índice del mensaje a borrar. Debe ser mayor o igual que 1.
        """

        typ, response = self.connection.store(str(index), '+FLAGS', r'(\Deleted)')
        typ, response = self.connection.expunge()
