# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, email, poplib, time, excepts

def fix_text(text):
    """
    Elimina los saltos de línea de una cadena de texto y sustituye cada
    tabulador por un espacio en blanco.

    `text` es la cadena de texto a tratar.
    """

    newText = text.replace("\n", "")
    newText = newText.replace("\t", " ")
    return newText

class EmailAttachment:
    """
    Clase que representa un archivo adjunto de un mensaje de e-mail de un servidor de POP.
    """

    def __init__(self, content, fileName):
        """
        Constructor de la clase.

        `content` es el contenido del archivo.

        `fileName` es el nombre del archivo.
        """

        self.content = content
        self.fileName = fix_text(fileName)
    
    def save(self, dirPath, fileName=None):
        """
        Guarda el archivo.

        `dirPath` es la ruta del directorio donde guardar el archivo.

        `fileName` es el nombre del archivo que se va a establecer. Si no se especifica, el nombre 
        del archivo es su nombre original.
        """

        if (fileName == None):
            path = os.path.join(dirPath, self.fileName)
        else:
            path = os.path.join(dirPath, fileName)
        outputFile = open(path, "wb")
        outputFile.write(self.content)
        outputFile.close()

class EmailMessage:
    """
    Clase que representa un mensaje de e-mail completo de un servidor de POP.
    """

    def __init__(self, messageNum, content):
        """
        Constructor de la clase.
        
        `messageNum` es el índice del mensaje dentro del contexto del servidor de e-mail.

        `content` es el contenido del mensaje.
        """

        self.messageNum = messageNum
        self.content = content
        self.attachmentsIndex = 0 
        self.attachments = []
        
        self.msglines = '\n'.join(content[1])
        self.msg = email.Parser.Parser().parsestr(self.msglines)
        if self.msg.is_multipart():
            attachmentNum = 0
            for part in self.msg.walk():
                mptype = part.get_content_maintype()
                filename = part.get_filename()
                if mptype == "multipart": continue
                if filename:
                    attachmentNum += 1
                    self.attachments.append(EmailAttachment(part.get_payload(decode=1), filename))
                else:
                    self.body = part.get_payload()
        else:
            self.body = self.msg.get_payload()

    def get(self, key):
        """
        Método necesario para que la clase sea iterable.

        Se itera sobre los archivos adjuntos.
        """

        return self.msg.get(key)

    def has_attachments(self):
        """
        Devuelve 'True' si el mensaje tiene archivos adjuntos. En caso contrario, devuelve `False`.
        """

        return (len(self.attachments) > 0)

    def __iter__(self):
        """
        Método necesario para que la clase sea iterable.

        Se itera sobre los archivos adjuntos.
        """

        return self

    def next(self):
        """
        Método necesario para que la clase sea iterable.

        Se itera sobre los archivos adjuntos.

        Devuelve el siguiente archivo adjunto (objeto de tipo `EmailAttachment`).
        """

        try: 
            attachment = self.attachments[self.attachmentsIndex]
        except:
            self.attachmentsIndex = 0
            raise StopIteration
        self.attachmentsIndex += 1
        return attachment

class EmailMessagePreview:
    """
    Clase que representa las cabeceras de un mensaje de e-mail de un servidor de POP.
    """

    def __init__(self, index, data):
        """
        Constructor de la clase.

        `index` es el índice del mensaje dentro del contexto del servidor de e-mail.

        `data` es el contenido de las cabeceras.
        """

        self.index = index
        elementCount = len(data)
        out = ""
        
        for i in range(elementCount):
            out += data[i]
            out += "\n"
        message = email.message_from_string(out)
        
        self.subject = fix_text(message["Subject"])
        
        fromStr = message["From"]
        a = fromStr.find("<")
        b = fromStr.rfind(">")
        if a != -1 and b != -1:
            self.fromAdd = fromStr[a + 1:b]
        else:
            self.fromAdd = fromStr

    def __cmp__(self, other):
        """
        Método comparador de la clase.

        Es necesario para saber si un mensaje es distinto de otro. El criterio de
        comparación es por su asunto.
        """

        if self.subject == other.subject:
            result = 0
        elif self.subject < other.subject:
            result = -1
        else:
            result = 1
        return result

class PopAccount:
    """
    Clase básica para acceder a una cuenta de un servidor de POP y descargar y borrar mensajes de e-mail.
    """

    lastProgress = -1
    
    def __init__(self, server, ssl, port, user, password):
        """
        Constructor de la clase.

        `server` es la dirección del servidor con el que conectar.

        `ssl` es una variable de tipo lógico que especifica si se debe usar una conexión
        segura por medio de SSL (depende del servidor).

        `port` es el puerto a usar para realizar la conexión (depende del servidor).

        `user` es el nombre de usuario con el que conectarse al servidor.

        `password` es la contraseña del usuario `user`.
        """

        self.server = server
        self.ssl = ssl
        self.port = port
        self.user = user
        self.password = password
        
    def connect(self):
        """
        Realiza la conexión con el servidor.
        """

        try:
            if self.ssl:
                self.connection = poplib.POP3_SSL(self.server, self.port)
            else:
                self.connection = poplib.POP3(self.server, self.port)
            self.connection.user(self.user)
            self.connection.pass_(self.password)
        except:
            self.close()

    def close(self):
        """
        Cierra la conexión con el servidor.

        Para llamar a este método, debe haber una conexión abierta.
        """

        if self.connection != None:
            self.connection.quit()
            self.connection = None

    def is_connection_ok(self):
        """
        Devuelve `True` si hay una conexión abierta y está operativa. En caso contrario, devuelve `False`.
        """

        ok = False
        
        try:
            self.connection.stat()
            ok = True
        except:
            pass
        return ok
        
    def get_messages_count(self):
        """
        Devuelve el número de mensajes de la cuenta.
        """

        return self.connection.stat()[0]

    # "index" starts at 1
    def get_message_preview(self, index):
        """
        Devuelve un objeto de tipo `pop.EmailMessagePreview`, representando a un mensaje
        de e-mail concreto.

        `index` es el índice del mensaje a obtener. Debe ser mayor o igual que 1.    
        """

        if index < 1 or index > self.get_messages_count():
            raise excepts.IndexException()
        header = self.connection.top(index, 1)[1]
        return EmailMessagePreview(index, header)
    
    # "index" starts at 1
    def get_message(self, index):
        """
        Devuelve un objeto de tipo `pop.EmailMessage`, representando a un mensaje
        de e-mail concreto.

        `index` es el índice del mensaje a obtener. Debe ser mayor o igual que 1.    
        """

        if index < 1 or index > self.get_messages_count():
            raise excepts.IndexException()
        return EmailMessage(index, self.connection.retr(index))
    
    # "index" starts at 1
    def remove_message(self, index):
        """
        Borra un mensaje de e-mail concreto.

        `index` es el índice del mensaje a borrar. Debe ser mayor o igual que 1.
        """

        if index < 1 or index > self.get_messages_count():
            raise excepts.IndexException()
        self.connection.dele(index)
