# -*- coding: utf-8 -*-

# Unimail 0.13 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, time, excepts, sysutils, syspaths

class PopLogic:
    """
    Clase que encapsula la lógica a la hora de recibir un archivo por medio de POP.

    Utiliza la clase básica `PopAccount`.
    """

    tempDir = syspaths.get_temp_input_dir()
    inboxData = []

    def __init__(self, account):
        """
        Constructor de la clase.

        `account` es un objeto de tipo `PopAccount`. Se utilizará para descargar
        cada una de las partes del archivo total a descargar.
        """

        self.account = account
    
    def refresh(self):
        """
        Actualiza la información sobre la bandeja de entrada del servidor.

        Introduce en `inboxData` las cabeceras de los mensajes de e-mail
        que han sido recibidos por Unimail.
        """

        yield 0
        self.inboxData = []
        msgCount = self.account.get_messages_count()

        for i in range(1, msgCount + 1):
            fail = True
            tries = 0
            while fail and tries < 5:
                try:                    
                    msgPreview = self.account.get_message_preview(i)
                    fail = False
                    if (self.is_unimail_message(msgPreview.subject)):
                        self.inboxData.append(msgPreview)
                    yield ((100*i)/msgCount)
                except:
                    self.account.close()
                    self.account.connect()
                tries += 1
            if fail:
                raise excepts.SearchException("Error searching")
        if msgCount == 0:
            yield 100

    # Must call function "refresh" first 
    def get_unimail_files_list(self):
        """
        Devuelve una lista con información básica de cada archivo recibido por medio de Unimail.

        Para llamar a este método, primero, debe haberse llamado al método `refresh`.
        """

        fileList = []
        checked = []
        
        for i in range(0, len(self.inboxData)):
            messagePrev = self.inboxData[i]
            ind = -1 
            try:
                ind = checked.index(messagePrev)
            except:
                pass

            if ind == -1 and self.is_complete_file(i):
                checked.append(messagePrev)
                fileList.append(self.get_unimail_message_info(i, messagePrev))
        
        return fileList
    
    # Returns information about an Unimail message:
    def get_unimail_message_info(self, index, messagePreview):
        """
        Devuelve información básica sobre un mensaje concreto de la bandeja de entrada
        que haya sido enviado mediante Unimail. Dicho mensaje es una parte de un archivo
        recibido que ha sido enviado por Unimail.

        La información que devuelve son, aparte del índice del mensaje, el asunto,
        la hora (en formato Unix) a la que se empezó a enviar, el número de partes
        del archivo al cual pertenece el mensaje (que es una de dichas partes), la dirección
        que ha enviado el mensaje y el nombre del archivo recibido.

        La información se devuelve en forma de diccionario.

        `index` es el índice del mensaje dentro del contexto del servidor de e-mail.

        `messagePreview` es un objeto de tipo `pop.EmailMessagePreview` con las cabeceras
        del mensaje del que se quiere obtener la información.
        """

        info = {}
        info["index"] = index
        info["from"] = messagePreview.fromAdd
        subject = messagePreview.subject
        posT1 = subject.find("[t]")
        posT2 = subject.rfind("[t]")
        posP1 = subject.find("[p]")
        posP2 = subject.rfind("[p]")
        info["time"] = subject[posT1+3:posT2]
        info["parts"] = int(subject[posP1+3:posP2])
        info["file"] = subject[posP2+3:]
        
        return info
            
    # Returns if a message it's a valid Unimail part message by its subject
    def is_unimail_message(self, subject):
        """
        Devuelve `True` si un mensaje es una parte válida de un archivo
        enviado por Unimail. Devuelve `False` en caso contrario.

        Se comprueba mirando si el asunto tiene o no el formato de Unimail.

        `subject` es el asunto del mensaje.
        """

        unimailMsg = False
        
        try:
            beg = subject[:16]
            posT1 = subject.find("[t]")
            posT2 = subject.rfind("[t]")
            posP1 = subject.find("[p]")
            posP2 = subject.rfind("[p]")
            time = float(subject[posT1+3:posT2])
            partCount = int(subject[posP1+3:posP2])
            fileName = subject[posP2+3:]
            
            unimailMsg = partCount > 0
            unimailMsg = unimailMsg and (fileName != "")
            unimailMsg = unimailMsg and ((beg == "[unimail-msg][t]") and (posT2 >= 16))
            unimailMsg = unimailMsg and ((posP1 == (posT2 + 3)) and (posP2 >= (posP1 + 4)))
        except:
            pass

        return unimailMsg
    
    # Returns if a received file is complete
    # Must call function "refresh" first
    def is_complete_file(self, index):
        """
        Devuelve 'True' si un archivo concreto ha sido recibido de forma completa
        (si se han recibido todas sus partes). Devuelve 'False' en caso contrario.

        `index` es el índice respecto a `inboxData` de cualquiera de las partes (mensajes) del archivo
        recibido a comprobar. Debe ser igual o mayor que 0.

        Para llamar a este método, primero, debe haberse llamado al método `refresh`.
        """

        if index < 0 or index > (len(self.inboxData) - 1):
            raise excepts.IndexException("Index not valid.")

        messagePrev = self.inboxData[index]
        info = self.get_unimail_message_info(index, messagePrev)
        
        return (self.inboxData.count(messagePrev) == info["parts"])

    # Must call function "refresh" first
    def download_file(self, index):
        """
        Descarga un archivo recibido que ha sido enviado por Unimail (descarga todas sus partes).

        `index` es el índice respecto a `inboxData` de cualquiera de las partes (mensajes) del archivo
        recibido a descargar. Debe ser igual o mayor que 0.

        Las partes se descargan en la carpeta de bandeja de entrada.

        Para llamar a este método, primero, debe haberse llamado al método `refresh`.
        """

        if index < 0 or index > (len(self.inboxData) - 1):
            raise excepts.IndexException("Index not valid.")
        # Check if the file is complete (if all parts have been received):
        if not self.is_complete_file(index):
            raise excepts.NotCompleteFileException("File is not complete.")
        
        yield 0

        messagePrev = self.inboxData[index]
        info = self.get_unimail_message_info(index, messagePrev)
        partCount = info["parts"]
        inboxIndex = messagePrev.index
        
        msgCount = len(self.inboxData)
        i = 0
        found = False
        part = 0
        
        while (i < msgCount and not found):
            messagePrev2 = self.inboxData[i]
            inboxIndex2 = messagePrev2.index
            
            if messagePrev2 == messagePrev:
                error = True
                tr = 0
                while error and tr < 6:
                    try:
                        message = self.account.get_message(inboxIndex2)
                        error = False
                    except:
                        tr += 1
                        if tr == 6:
                            raise excepts.ConnectionException("Error downloading file")
                        self.account.connect()
                        self.refresh()
                        error = True
                        
                if message.has_attachments():
                    (message.next()).save(self.tempDir)
                part += 1
                if part == partCount:
                    found = True
                yield ((100*part)/partCount)
            i += 1
